import 'package:yflutter_widgets/widgets/listview/model/ylvsectionmodel.dart';
import 'package:yflutter_widgets/widgets/listview/ylistviewbody.dart';
import 'package:yflutter_widgets/widgets/listview/ylvconfigs.dart';
import 'package:yflutter_widgets/widgets/listview/ylvheaders/ylvappbarheader.dart';
import 'package:yflutter_widgets/widgets/listview/ylvheaders/ylvzoomheader.dart';
import 'package:yflutter_widgets/widgets/listview/ylvproviders/contractedbarprovider.dart';
import 'package:yflutter_widgets/widgets/ycustom_card.dart';
import 'package:flutter/material.dart';
import 'package:provider/provider.dart';

/// ListView Header Type
/// None: No header, as Default,
/// AppBar: Provide a appbar level header with contracted/zoom feature,
/// * Do not set AppBar in your view/page when use this type, cuz it already provides,
/// * set height of it,
/// Zoom: A Header with contracted/zoom feature, under app bar, set height(max) and min height,
/// Normal: A scrollable header, which means will be out of view when scroll, set height only, default 150.
enum HeaderType {
  None,
  AppBar,
  Zoom,
  Normal,
}

/// Boarder of ListView by YCustomCard style
/// None: no boarder, as Default,
/// ListWhole, set boarder for whole ListView including Header,
/// ListSection, set boarder for each section.
///
/// TODO: Next Step, add boarder for whole ListView except Header.
enum BorderType {
  None,
  ListWhole,
  ListSection,
}

class YListView extends StatefulWidget {
  YListView({
    this.type = HeaderType.None,
    this.borderType = BorderType.None,
    this.borderMargin =
        const EdgeInsets.only(left: 16, top: 16, right: 16, bottom: 16),
    this.title = "",
    this.listHeader,
    this.headerHeight = 150.0,
    this.headerMinHeight = 44.0,
    @required this.numOfSection,
    @required this.numberOfRowsInSection,
    @required this.sectionHeight,
    @required this.cellHeight,
    @required this.sectionHeader,
    @required this.cell,
    this.sectionHeadersPinToVisibleBounds = false,
  });

  final HeaderType type;
  final BorderType borderType;
  final EdgeInsets borderMargin;

  // Title of the AppBar Type Header.
  final String title;

  // Header Content for ListView Header when HeaderType is not None.
  final Widget listHeader;

  // Max/Normal Height of the Header.
  final double headerHeight;

  // Min Height of the Header, only when it can be contracted.
  final double headerMinHeight;

  final int numOfSection;
  final NumberOfRowsInSection numberOfRowsInSection;
  final SectionHeight sectionHeight;
  final CellHeight cellHeight;
  final HeaderOfSectionAtIndex sectionHeader;
  final CellOfRowAtIndex cell;

  // Section Header Pin feature, default false.
  final bool sectionHeadersPinToVisibleBounds;

  @override
  _YListViewState createState() => _YListViewState();
}

class _YListViewState extends State<YListView> {
  ScrollController _scrollController;
  ContractedBarProvider _contractedBarProvider = ContractedBarProvider();
  bool lastContractedStatus = false;

  List<YLVSectionModel> sections = [];
  List<Widget> total = [];

  bool get isContracted {
    return _scrollController.hasClients &&
        _scrollController.offset > (180 - kToolbarHeight);
  }

  // Not use now.
  _scrollListener() {
    if (lastContractedStatus != isContracted) {
      lastContractedStatus = isContracted;
      _contractedBarProvider.contractStatusChange(isContracted);
    }
  }

  @override
  void initState() {
    _scrollController = ScrollController();
    _scrollController.addListener(_scrollListener);
    initBaseData();
    super.initState();
  }

  @override
  void dispose() {
    _scrollController.removeListener(_scrollListener);
    _scrollController.dispose();
    _contractedBarProvider.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return ChangeNotifierProvider<ContractedBarProvider>.value(
        value: _contractedBarProvider,
        child: NestedScrollView(
          headerSliverBuilder: (BuildContext context, bool innerBoxIsScrolled) {
            return [
              // if (widget.type != HeaderType.None)
              SliverOverlapAbsorber(
                handle:
                    NestedScrollView.sliverOverlapAbsorberHandleFor(context),
                sliver: zoomHeaderConfig(),
              ),
            ];
          },
          controller: _scrollController,
          body: Builder(
            builder: (BuildContext context) {
              return widget.borderType == BorderType.ListWhole
                  ? YCustomCard(
                      insideWidget: CustomScrollView(
                        // controller: _scrollController,
                        slivers: [
                          if (widget.type != HeaderType.None)
                            SliverOverlapInjector(
                              // This is the flip side of the SliverOverlapAbsorber
                              // above.
                              handle:
                                  NestedScrollView.sliverOverlapAbsorberHandleFor(
                                      context),
                            ),
                          ...total
                        ],
                      ),
                      sEdgeInsets: widget.borderMargin)
                  : CustomScrollView(
                      // controller: _scrollController,
                      slivers: [
                        if (widget.type != HeaderType.None)
                          SliverOverlapInjector(
                            // This is the flip side of the SliverOverlapAbsorber
                            // above.
                            handle:
                                NestedScrollView.sliverOverlapAbsorberHandleFor(
                                    context),
                          ),
                        ...total
                      ],
                    );
            },
          ),
        ));
  }

  /// Setup ListVie Header
  Widget zoomHeaderConfig() {
    // AppBar Header
    if (widget.type == HeaderType.AppBar)
      return YLVAppBarHeader(
        title: widget.title,
        headerHeight: widget.headerHeight,
        insideWidget: Row(
          children: [Expanded(child: widget.listHeader)],
        ),
      );
    // Zoom Header
    if (widget.type == HeaderType.Zoom)
      return YLVZoomHeader(
        insideWidget: widget.listHeader,
        minHeight: widget.headerMinHeight,
        maxHeight: widget.headerHeight,
        pinned: true,
      );
    // Normal Header
    else
      return YLVZoomHeader(
        insideWidget: widget.listHeader,
        minHeight: widget.headerHeight,
        maxHeight: widget.headerHeight,
        pinned: false,
      );
  }

  /// Setup ListView Content
  void initBaseData() {
    this.sections.clear();
    this.total.clear();

    for (int sectionIndex = 0;
        sectionIndex < widget.numOfSection;
        sectionIndex++) {
      int rowCount = widget.numberOfRowsInSection(sectionIndex);
      Widget sectionHeader = widget.sectionHeader(context, sectionIndex);
      double sectionHeight;
      if (sectionHeader != null) {
        sectionHeight = this.widget.sectionHeight(context, sectionIndex);
      } else {
        sectionHeight = 0;
      }
      if (!widget.sectionHeadersPinToVisibleBounds) {
        sectionHeader = SliverToBoxAdapter(
            child: widget.sectionHeader(context, sectionIndex));
      } else {
        sectionHeader = YLVZoomHeader(
          insideWidget: widget.sectionHeader(context, sectionIndex),
          minHeight: sectionHeight,
          maxHeight: sectionHeight,
        );
      }

      YLVSectionModel model = YLVSectionModel(
        height: sectionHeight,
        section: sectionIndex,
        headerWidget: sectionHeader,
      );
      sections.add(model);

      // Add Header for each section
      total.add(sectionHeader);
      // Add Section Body
      total.add(YLVBody(
        cell: widget.cell,
        numRow: rowCount,
        sectionIndex: sectionIndex,
        hasBoarder: widget.borderType == BorderType.ListSection,
      ));
    }
  }
}
